# Generated by sila2.code_generator; sila2.__version__: 0.12.2
from __future__ import annotations

import uuid
from calendar import LocaleTextCalendar
from typing import TYPE_CHECKING

from sila2.framework.data_types.any import SilaAnyType
from sila2.server import MetadataDict

from ..generated.labware import (
    DeleteLabware_Responses,
    LabwareBase,
    MoveLabware_Responses,
    RegisterLabware_Responses,
)

if TYPE_CHECKING:
    from ..server import Server

from ..base import Base

from ..labware.labware_wrapper import LabwareWrapper


class LabwareImpl(LabwareBase):
    def __init__(self, parent_server: Server, base: Base) -> None:
        super().__init__(parent_server=parent_server)
        self.base = base

    def RegisterLabware(
            self, SessionIdentifier: str, LoadName: str, Location: str, *,
            metadata: MetadataDict
    ) -> RegisterLabware_Responses:
        if self.base.get_session_id() != SessionIdentifier:
            raise "Wrong Identifier"

        new_item_identifier = str(uuid.uuid4())

        if Location.isdigit():
            new_labware = self.base.protocol.load_labware(LoadName,
                                                          int(Location),
                                                          new_item_identifier)
        else:
            new_labware = self.base.protocol.load_labware(LoadName, Location,
                                                          new_item_identifier)

        self.base.add_labware(LabwareWrapper(new_labware, Location),
                              new_item_identifier)
        return RegisterLabware_Responses(ItemIdentifier=new_item_identifier)

    def DeleteLabware(
            self, SessionIdentifier: str, Identifier: str, *,
            metadata: MetadataDict
    ) -> DeleteLabware_Responses:
        if self.base.get_session_id() != SessionIdentifier:
            raise "Wrong Identifier"

        if uuid.UUID(Identifier).version == 4:
            labware = self.base.get_labware(Identifier)
        else:
            labware = self.base.get_labware_by_position(Identifier)

        for key, value in self.base.protocol.loaded_labwares.items():
            if value == labware:
                del self.base.protocol.loaded_labwares[key]
        self.base.delete_labware(Identifier)
        return DeleteLabware_Responses()

    def MoveLabware(
            self, SessionIdentifier: str, ItemIdentifier: str, Location: str, *,
            metadata: MetadataDict
    ) -> MoveLabware_Responses:
        if self.base.get_session_id() != SessionIdentifier:
            raise "Wrong Identifier"
        for key, value in self.base.labwares.items():
            if key == ItemIdentifier:
                if Location.isdigit():
                    self.base.protocol.move_labware(labware=value.labware,
                                                    new_location=int(Location),
                                                    use_gripper=True)
                else:
                    self.base.protocol.move_labware(labware=value.labware,
                                                    new_location=Location,
                                                    use_gripper=True)

        return MoveLabware_Responses()



# Generated by sila2.code_generator; sila2.__version__: 0.12.2
# -----
# This class does not do anything useful at runtime. Its only purpose is to provide type annotations.
# Since sphinx does not support .pyi files (yet?), this is a .py file.
# -----

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    

    

    
    
    
    from typing import Optional, Iterable
    from sila2.client import ClientMetadataInstance
    
    
    from labware_types import RegisterLabware_Responses
    
    from labware_types import DeleteLabware_Responses
    
    from labware_types import MoveLabware_Responses
    
    

    

    


class LabwareClient:
    """
    This feature allows you to manipulate the labware in the OT-2 and
        OT-Flex
    """
    
    

    

    

    
    def RegisterLabware(
        self,
        
        SessionIdentifier: str,
        
        LoadName: str,
        
        Location: str,
        
        *,
        metadata: Optional[Iterable[ClientMetadataInstance]] = None
    ) -> RegisterLabware_Responses:
        """
        This method registers labware to a specified position for
            the current session
        """
        ...
    
    def DeleteLabware(
        self,
        
        SessionIdentifier: str,
        
        Identifier: str,
        
        *,
        metadata: Optional[Iterable[ClientMetadataInstance]] = None
    ) -> DeleteLabware_Responses:
        """
        This method deletes labware without removing it, Only use this
            method if you are removing it yourself and make sure to not execute
            any commands while moving it.
        """
        ...
    
    def MoveLabware(
        self,
        
        SessionIdentifier: str,
        
        ItemIdentifier: str,
        
        Location: str,
        
        *,
        metadata: Optional[Iterable[ClientMetadataInstance]] = None
    ) -> MoveLabware_Responses:
        """
        This method deletes labware without removing it, Only use this
            method if you are removing it yourself and make sure to not execute
            any commands while moving it.
        """
        ...
    

    



# Generated by sila2.code_generator; sila2.__version__: 0.12.2

from typing import Optional
from uuid import UUID, uuid4

from sila2.server import SilaServer

from .feature_implementations.labware_impl import LabwareImpl
from .feature_implementations.sessionmanagement_impl import SessionManagementImpl
from .generated.labware import LabwareFeature
from .generated.sessionmanagement import SessionManagementFeature

from .base import Base


class Server(SilaServer):
    def __init__(
        self,
        server_uuid: Optional[UUID] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
    ):
        if name is None:
            name = "Opentrons 2 or Opentrons Flex"
        if description is None:
            description = "Server for the Opentrons 2 or Opentrons Flex"
        super().__init__(
            server_name=name,
            server_description=description,
            server_type="Pipetting",
            server_version="0.2.0_beta",
            server_vendor_url="https://gitlab.com/SiLA2/sila_python",
            server_uuid=server_uuid if server_uuid is not None else uuid4(),
        )

        self.base = Base()

        self.sessionmanagement = SessionManagementImpl(self, self.base, simulate=False)
        self.set_feature_implementation(SessionManagementFeature, self.sessionmanagement)

        self.labware = LabwareImpl(self, self.base)
        self.set_feature_implementation(LabwareFeature, self.labware)
